@@ -74,8 +74,10 @@ WinMain (struct HINSTANCE__ *hInstance,
   int fd;
   int mode;
   int handle;
+  int saved_errno;
   int no_error = CHILD_NO_ERROR;
   int zero = 0;
+  gint file_and_argv_zero = 0;
   gchar **new_argv;
 
   SETUP_DEBUG();
@@ -105,6 +107,10 @@ WinMain (struct HINSTANCE__ *hInstance,
    */
   child_err_report_fd = atoi (__argv[ARG_CHILD_ERR_REPORT]);
 
+  /* Hack to implement G_SPAWN_FILE_AND_ARGV_ZERO */
+  if (__argv[ARG_CHILD_ERR_REPORT][strlen (__argv[ARG_CHILD_ERR_REPORT]) - 1] == '#')
+    file_and_argv_zero = 1;
+
   /* argv[ARG_STDIN..ARG_STDERR] are the file descriptors that should
    * be dup2'd to stdin, stdout and stderr, '-' if the corresponding
    * std* should be let alone, and 'z' if it should be connected to
@@ -220,7 +226,7 @@ WinMain (struct HINSTANCE__ *hInstance,
 					__argv[ARG_PROGRAM],
 					(mode == P_WAIT ?
 					 "P_WAIT" : "P_NOWAIT")));
-      i = ARG_PROGRAM+1;
+      i = ARG_PROGRAM + 1 + file_and_argv_zero;
       while (new_argv[i])
 	{
 	  g_string_append (debugstring, new_argv[i++]);
@@ -231,9 +237,11 @@ WinMain (struct HINSTANCE__ *hInstance,
     }
 
   if (new_argv[ARG_USE_PATH][0] == 'y')
-    handle = spawnvp (mode, __argv[ARG_PROGRAM], new_argv+ARG_PROGRAM);
+    handle = spawnvp (mode, __argv[ARG_PROGRAM], new_argv + ARG_PROGRAM + file_and_argv_zero);
   else
-    handle = spawnv (mode, __argv[ARG_PROGRAM], new_argv+ARG_PROGRAM);
+    handle = spawnv (mode, __argv[ARG_PROGRAM], new_argv + ARG_PROGRAM + file_and_argv_zero);
+
+  saved_errno = errno;
 
   if (debug)
     {
@@ -246,7 +254,7 @@ WinMain (struct HINSTANCE__ *hInstance,
       MessageBox (NULL, debugstring->str, "gspawn-win32-helper", 0);
     }
 
-  if (handle < 0)
+  if (handle == -1 && saved_errno != 0)
     write_err_and_exit (child_err_report_fd, CHILD_SPAWN_FAILED);
 
   write (child_err_report_fd, &no_error, sizeof (no_error));
