@@ -64,7 +64,6 @@ static gchar*	value_param_lcopy_value		(const GValue	*value,
 
 /* --- variables --- */
 static GQuark quark_floating = 0;
-G_LOCK_DEFINE_STATIC (pspec_ref_count);
 
 
 /* --- functions --- */
@@ -152,10 +151,15 @@ static void
 g_param_spec_finalize (GParamSpec *pspec)
 {
   g_datalist_clear (&pspec->qdata);
+
+  if (!(pspec->flags & G_PARAM_STATIC_NAME))
+    g_free (pspec->name);
   
-  g_free (pspec->name);
-  g_free (pspec->_nick);
-  g_free (pspec->_blurb);
+  if (!(pspec->flags & G_PARAM_STATIC_NICK))
+    g_free (pspec->_nick);
+
+  if (!(pspec->flags & G_PARAM_STATIC_BLURB))
+    g_free (pspec->_blurb);
 
   g_type_free_instance ((GTypeInstance*) pspec);
 }
@@ -164,43 +168,26 @@ GParamSpec*
 g_param_spec_ref (GParamSpec *pspec)
 {
   g_return_val_if_fail (G_IS_PARAM_SPEC (pspec), NULL);
+  g_return_val_if_fail (pspec->ref_count > 0, NULL);
+
+  g_atomic_int_inc (&pspec->ref_count);
 
-  G_LOCK (pspec_ref_count);
-  if (pspec->ref_count > 0)
-    {
-      pspec->ref_count += 1;
-      G_UNLOCK (pspec_ref_count);
-    }
-  else
-    {
-      G_UNLOCK (pspec_ref_count);
-      g_return_val_if_fail (pspec->ref_count > 0, NULL);
-    }
-  
   return pspec;
 }
 
 void
 g_param_spec_unref (GParamSpec *pspec)
 {
+  gboolean is_zero;
+
   g_return_if_fail (G_IS_PARAM_SPEC (pspec));
+  g_return_if_fail (pspec->ref_count > 0);
 
-  G_LOCK (pspec_ref_count);
-  if (pspec->ref_count > 0)
-    {
-      gboolean need_finalize;
+  is_zero = g_atomic_int_dec_and_test (&pspec->ref_count);
 
-      /* sync with _sink */
-      pspec->ref_count -= 1;
-      need_finalize = pspec->ref_count == 0;
-      G_UNLOCK (pspec_ref_count);
-      if (need_finalize)
-	G_PARAM_SPEC_GET_CLASS (pspec)->finalize (pspec);
-    }
-  else
+  if (G_UNLIKELY (is_zero))
     {
-      G_UNLOCK (pspec_ref_count);
-      g_return_if_fail (pspec->ref_count > 0);
+      G_PARAM_SPEC_GET_CLASS (pspec)->finalize (pspec);
     }
 }
 
@@ -208,30 +195,10 @@ void
 g_param_spec_sink (GParamSpec *pspec)
 {
   g_return_if_fail (G_IS_PARAM_SPEC (pspec));
+  g_return_if_fail (pspec->ref_count > 0);
 
-  G_LOCK (pspec_ref_count);
-  if (pspec->ref_count > 0)
-    {
-      if (g_datalist_id_remove_no_notify (&pspec->qdata, quark_floating))
-	{
-	  /* sync with _unref */
-	  if (pspec->ref_count > 1)
-	    pspec->ref_count -= 1;
-	  else
-	    {
-	      G_UNLOCK (pspec_ref_count);
-	      g_param_spec_unref (pspec);
-
-	      return;
-	    }
-	}
-      G_UNLOCK (pspec_ref_count);
-    }
-  else
-    {
-      G_UNLOCK (pspec_ref_count);
-      g_return_if_fail (pspec->ref_count > 0);
-    }
+  if (g_datalist_id_remove_no_notify (&pspec->qdata, quark_floating))
+    g_param_spec_unref (pspec);
 }
 
 G_CONST_RETURN gchar*
@@ -297,6 +264,25 @@ canonicalize_key (gchar *key)
     }
 }
 
+static gboolean
+is_canonical (const gchar *key)
+{
+  const gchar *p;
+
+  for (p = key; *p != 0; p++)
+    {
+      gchar c = *p;
+      
+      if (c != '-' &&
+	  (c < '0' || c > '9') &&
+	  (c < 'A' || c > 'Z') &&
+	  (c < 'a' || c > 'z'))
+	return FALSE;
+    }
+
+  return TRUE;
+}
+
 gpointer
 g_param_spec_internal (GType        param_type,
 		       const gchar *name,
@@ -309,12 +295,28 @@ g_param_spec_internal (GType        param_type,
   g_return_val_if_fail (G_TYPE_IS_PARAM (param_type) && param_type != G_TYPE_PARAM, NULL);
   g_return_val_if_fail (name != NULL, NULL);
   g_return_val_if_fail ((name[0] >= 'A' && name[0] <= 'Z') || (name[0] >= 'a' && name[0] <= 'z'), NULL);
+  g_return_val_if_fail (!(flags & G_PARAM_STATIC_NAME) || is_canonical (name), NULL);
   
   pspec = (gpointer) g_type_create_instance (param_type);
-  pspec->name = g_strdup (name);
-  canonicalize_key (pspec->name);
-  pspec->_nick = g_strdup (nick);
-  pspec->_blurb = g_strdup (blurb);
+
+  if ((flags & G_PARAM_STATIC_NAME))
+    pspec->name = (gchar *) name;
+  else
+    {
+      pspec->name = g_strdup (name);
+      canonicalize_key (pspec->name);
+    }
+
+  if (flags & G_PARAM_STATIC_NICK)
+    pspec->_nick = (gchar *) nick;
+  else
+    pspec->_nick = g_strdup (nick);
+
+  if (flags & G_PARAM_STATIC_BLURB)
+    pspec->_blurb = (gchar *) blurb;
+  else
+    pspec->_blurb = g_strdup (blurb);
+
   pspec->flags = (flags & G_PARAM_USER_MASK) | (flags & G_PARAM_MASK);
   
   return pspec;
@@ -699,7 +701,7 @@ param_spec_ht_lookup (GHashTable  *hash_table,
   else
     pspec = g_hash_table_lookup (hash_table, &key);
 
-  if (!pspec)
+  if (!pspec && !is_canonical (param_name))
     {
       /* try canonicalized form */
       key.name = g_strdup (param_name);
